@@ -16,9 +16,22 @@
 using namespace KOSMIndoorMap;
 using namespace KPublicTransport;
 
+struct vehicle_type {
+    const char *tagName;
+    RentalVehicle::VehicleType vehicleType;
+};
+static constexpr const vehicle_type vehicle_type_map[] = {
+    { "mx:realtime_available:bike", RentalVehicle::Bicycle },
+    { "mx:realtime_available:pedelec", RentalVehicle::Pedelec },
+    { "mx:realtime_available:scooter", RentalVehicle::ElectricKickScooter },
+    { "mx:realtime_available:motorcycle", RentalVehicle::ElectricMoped },
+    { "mx:realtime_available:car", RentalVehicle::Car },
+};
+
 LocationQueryOverlayProxyModel::LocationQueryOverlayProxyModel(QObject *parent)
     : QAbstractListModel(parent)
 {
+    static_assert((sizeof(vehicle_type_map) / sizeof(vehicle_type)) == (sizeof(LocationQueryOverlayProxyModel::m_realtimeAvailableTagKeys) / sizeof(OSM::TagKey)));
 }
 
 LocationQueryOverlayProxyModel::~LocationQueryOverlayProxyModel() = default;
@@ -44,11 +57,18 @@ void LocationQueryOverlayProxyModel::setMapData(const MapData &data)
         m_tagKeys.realtimeAvailable = m_data.dataSet().makeTagKey("mx:realtime_available");
         m_tagKeys.network = m_data.dataSet().makeTagKey("network");
         m_tagKeys.mxoid = m_data.dataSet().makeTagKey("mx:oid");
+        m_tagKeys.remainingRange = m_data.dataSet().makeTagKey("mx:remaining_range");
+        m_tagKeys.vehicle = m_data.dataSet().makeTagKey("mx:vehicle");
+    }
+
+    int i = 0;
+    for (const auto &v : vehicle_type_map) {
+        m_realtimeAvailableTagKeys[i++] = m_data.dataSet().makeTagKey(v.tagName);
     }
 
     initialize();
     endResetModel();
-    emit mapDataChanged();
+    Q_EMIT mapDataChanged();
 }
 
 QObject* LocationQueryOverlayProxyModel::sourceModel() const
@@ -96,7 +116,7 @@ void LocationQueryOverlayProxyModel::setSourceModel(QObject *sourceModel)
         for (int i = first.row(); i <= last.row(); ++i) {
             m_nodes[i] = nodeForRow(i);
         }
-        emit dataChanged(index(first.row(), 0), index(last.row(), 0));
+        Q_EMIT dataChanged(index(first.row(), 0), index(last.row(), 0));
     });
 }
 
@@ -163,6 +183,9 @@ LocationQueryOverlayProxyModel::Info LocationQueryOverlayProxyModel::nodeForRow(
             Q_UNREACHABLE();
             break;
         case Location::RentedVehicleStation:
+        {
+            const auto station = loc.rentalVehicleStation();
+
             // try to find a matching node in the base OSM data
             for (const auto &n : m_data.dataSet().nodes) {
                 if (OSM::distance(n.coordinate, info.overlayNode.coordinate) < 10 && OSM::tagValue(n, m_tagKeys.amenity) == "bicycle_rental") {
@@ -176,24 +199,55 @@ LocationQueryOverlayProxyModel::Info LocationQueryOverlayProxyModel::nodeForRow(
 
             info.overlayNode.id = m_data.dataSet().nextInternalId();
             OSM::setTagValue(info.overlayNode, m_tagKeys.amenity, "bicycle_rental");
-            if (loc.rentalVehicleStation().capacity() >= 0) {
-                OSM::setTagValue(info.overlayNode, m_tagKeys.capacity, QByteArray::number(loc.rentalVehicleStation().capacity()));
+            if (station.capacity() >= 0) {
+                OSM::setTagValue(info.overlayNode, m_tagKeys.capacity, QByteArray::number(station.capacity()));
             }
-            OSM::setTagValue(info.overlayNode, m_tagKeys.realtimeAvailable, QByteArray::number(loc.rentalVehicleStation().availableVehicles()));
-            if (OSM::tagValue(info.overlayNode, m_tagKeys.network).isEmpty() && !loc.rentalVehicleStation().network().name().isEmpty()) {
-                OSM::setTagValue(info.overlayNode, m_tagKeys.network, loc.rentalVehicleStation().network().name().toUtf8());
+            OSM::setTagValue(info.overlayNode, m_tagKeys.realtimeAvailable, QByteArray::number(station.availableVehicles()));
+            if (OSM::tagValue(info.overlayNode, m_tagKeys.network).isEmpty() && !station.network().name().isEmpty()) {
+                OSM::setTagValue(info.overlayNode, m_tagKeys.network, station.network().name().toUtf8());
             }
+
+            int i = 0;
+            for (const auto &v : vehicle_type_map) {
+                if (station.availableVehicles(v.vehicleType) > 0) {
+                    OSM::setTagValue(info.overlayNode, m_realtimeAvailableTagKeys[i], QByteArray::number(station.availableVehicles(v.vehicleType)));
+                }
+                ++i;
+            }
+
             break;
+        }
         case Location::RentedVehicle:
+        {
+            const auto vehicle = loc.data().value<RentalVehicle>();
+
             // free floating vehicles have no matching OSM element, so no point in searching for one
             info.overlayNode.id = m_data.dataSet().nextInternalId();
-            OSM::setTagValue(info.overlayNode, m_tagKeys.amenity, "bicycle_rental");
+            switch (vehicle.type()) {
+                case RentalVehicle::Unknown:
+                case RentalVehicle::Bicycle:
+                case RentalVehicle::Pedelec:
+                    OSM::setTagValue(info.overlayNode, m_tagKeys.vehicle, "bicycle_rental");
+                    break;
+                case RentalVehicle::ElectricKickScooter:
+                    OSM::setTagValue(info.overlayNode, m_tagKeys.vehicle, "scooter_rental");
+                    break;
+                case RentalVehicle::ElectricMoped:
+                    OSM::setTagValue(info.overlayNode, m_tagKeys.vehicle, "motorcycle_rental");
+                    break;
+                case RentalVehicle::Car:
+                    OSM::setTagValue(info.overlayNode, m_tagKeys.vehicle, "car_rental");
+                    break;
+            }
             OSM::setTagValue(info.overlayNode, m_tagKeys.name, loc.name().toUtf8());
-            OSM::setTagValue(info.overlayNode, m_tagKeys.realtimeAvailable, "1");
             if (OSM::tagValue(info.overlayNode, m_tagKeys.network).isEmpty() && !loc.rentalVehicle().network().name().isEmpty()) {
                 OSM::setTagValue(info.overlayNode, m_tagKeys.network, loc.rentalVehicle().network().name().toUtf8());
             }
+            if (vehicle.remainingRange() >= 0) {
+                OSM::setTagValue(info.overlayNode, m_tagKeys.remainingRange, QByteArray::number(vehicle.remainingRange()));
+            }
             break;
+        }
         case Location::Equipment:
             break;
     }
